==> akonadi-18.08.2.tar.xz: akonadi-18.08.2/src/core/asyncselectionhandler.cpp <==
/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "asyncselectionhandler_p.h"
#include "models/entitytreemodel.h"
#include "akonadicore_debug.h"

using namespace Akonadi;

AsyncSelectionHandler::AsyncSelectionHandler(QAbstractItemModel *model, QObject *parent)
    : QObject(parent)
    , mModel(model)
{
    Q_ASSERT(mModel);

    connect(mModel, &QAbstractItemModel::rowsInserted, this, &AsyncSelectionHandler::rowsInserted);
}

AsyncSelectionHandler::~AsyncSelectionHandler()
{
}

bool AsyncSelectionHandler::scanSubTree(const QModelIndex &index, bool searchForItem)
{
    if (searchForItem) {
        const Item::Id id = index.data(EntityTreeModel::ItemIdRole).toLongLong();

        if (mItem.id() == id) {
            emit itemAvailable(index);
            return true;
        }
    } else {
        const Collection::Id id = index.data(EntityTreeModel::CollectionIdRole).toLongLong();

        if (mCollection.id() == id) {
            emit collectionAvailable(index);
            return true;
        }
    }

    for (int row = 0; row < mModel->rowCount(index); ++row) {
        const QModelIndex childIndex = mModel->index(row, 0, index);
        //This should not normally happen, but if it does we end up in an endless loop
        if (!childIndex.isValid()) {
            qCWarning(AKONADICORE_LOG) << "Invalid child detected: " << index.data().toString();
            Q_ASSERT(false);
            return false;
        }
        if (scanSubTree(childIndex, searchForItem)) {
            return true;
        }
    }

    return false;
}

void AsyncSelectionHandler::waitForCollection(const Collection &collection)
{
    mCollection = collection;

    scanSubTree(QModelIndex(), false);
}

void AsyncSelectionHandler::waitForItem(const Item &item)
{
    mItem = item;

    scanSubTree(QModelIndex(), true);
}

void AsyncSelectionHandler::rowsInserted(const QModelIndex &parent, int start, int end)
{
    for (int i = start; i <= end; ++i) {
        scanSubTree(mModel->index(i, 0, parent), false);
        scanSubTree(mModel->index(i, 0, parent), true);
    }
}

#include "moc_asyncselectionhandler_p.cpp"

==> akonadi_19.12.3.orig.tar.xz: akonadi-19.12.3/src/core/asyncselectionhandler.cpp <==
/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "asyncselectionhandler_p.h"
#include "models/entitytreemodel.h"
#include "akonadicore_debug.h"

using namespace Akonadi;

AsyncSelectionHandler::AsyncSelectionHandler(QAbstractItemModel *model, QObject *parent)
    : QObject(parent)
    , mModel(model)
{
    Q_ASSERT(mModel);

    connect(mModel, &QAbstractItemModel::rowsInserted, this, &AsyncSelectionHandler::rowsInserted);
}

AsyncSelectionHandler::~AsyncSelectionHandler()
{
}

bool AsyncSelectionHandler::scanSubTree(const QModelIndex &index, bool searchForItem)
{
    if (searchForItem) {
        const Item::Id id = index.data(EntityTreeModel::ItemIdRole).toLongLong();

        if (mItem.id() == id) {
            Q_EMIT itemAvailable(index);
            return true;
        }
    } else {
        const Collection::Id id = index.data(EntityTreeModel::CollectionIdRole).toLongLong();

        if (mCollection.id() == id) {
            Q_EMIT collectionAvailable(index);
            return true;
        }
    }

    for (int row = 0; row < mModel->rowCount(index); ++row) {
        const QModelIndex childIndex = mModel->index(row, 0, index);
        //This should not normally happen, but if it does we end up in an endless loop
        if (!childIndex.isValid()) {
            qCWarning(AKONADICORE_LOG) << "Invalid child detected: " << index.data().toString();
            Q_ASSERT(false);
            return false;
        }
        if (scanSubTree(childIndex, searchForItem)) {
            return true;
        }
    }

    return false;
}

void AsyncSelectionHandler::waitForCollection(const Collection &collection)
{
    mCollection = collection;

    scanSubTree(QModelIndex(), false);
}

void AsyncSelectionHandler::waitForItem(const Item &item)
{
    mItem = item;

    scanSubTree(QModelIndex(), true);
}

void AsyncSelectionHandler::rowsInserted(const QModelIndex &parent, int start, int end)
{
    for (int i = start; i <= end; ++i) {
        scanSubTree(mModel->index(i, 0, parent), false);
        scanSubTree(mModel->index(i, 0, parent), true);
    }
}

#include "moc_asyncselectionhandler_p.cpp"
